# Twitcaspy
# Copyright 2021 Alma-field
# See LICENSE for details.

from .model import Model

class Result(Model):
    """API Result Object

    Attributes
    ----------
    latelimit : :class:`~twitcaspy.models.LateLimit`
    """
